
function [sys,x0,str,ts]=Grue(t,x,u,flag,Param)

switch flag

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0         
    [sys,x0,str,ts] = mdlInitializeSizes(Param);

  %%%%%%%%%%%%%%%
  % Derivatives %
  %%%%%%%%%%%%%%%
  case 1 
      
    sys = mdlDerivatives(t,x,u,Param);

  %%%%%%%%%%%%%%%%%%%%%%%%
  % Update and Terminate %
  %%%%%%%%%%%%%%%%%%%%%%%%
  case {2,9}
    sys = []; % do nothing
    

  %%%%%%%%%%
  % Output %
  %%%%%%%%%%
  case 3
      %keyboard
    sys = mdlOutputs(t,x,u); 

  otherwise
    error(['unhandled flag = ',num2str(flag)]);
end

% end limintm

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes(Param)

sizes = simsizes;
sizes.NumContStates  = 22;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 22;
sizes.NumInputs      = 9;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);
str = [];

x0  = Param.Xinit;
ts  = [0 0];   % sample time: [period, offset]

% end mdlInitializeSizes

%

%=============================================================================
% mdlDerivatives
% Compute derivatives for continuous states.
%=============================================================================
%
function sys = mdlDerivatives(t,x,u,Param)
m   = Param.m;
mo  = Param.mo;
m1  = Param.m1;
m2  = Param.m2;
m3  = Param.m3; 
m4  = Param.m4;
x11 = Param.x11;
x12 = Param.x12;
x13 = Param.x13;
x21 = Param.x21;
x22 = Param.x22; 
x23 = Param.x23;
x31 = Param.x31;
x32 = Param.x32;
x33 = Param.x33;
x41 = Param.x41;
x42 = Param.x42; 
x43 = Param.x43;

KP = Param.KP; 
KD = Param.KD;
g  = Param.g;

x1   = x(1);
x1d  = x(2);
x2   = x(3);
x2d  = x(4);
x3   = x(5);
x3d  = x(6);
x01  = x(7);
x01d = x(8);
x02  = x(9);
x02d = x(10);
x03  = x(11);
x03d = x(12);
L0   = x(13);
L0d  = x(14);
L1   = x(15);
L1d  = x(16);
L2   = x(17);
L2d  = x(18);
L3   = x(19);
L3d  = x(20);
L4   = x(21);
L4d  = x(22);

T1 = u(1);
T2 = u(2);
T3 = u(3);
T4 = u(4);


% Jose 
x1dd_dis = u(5);
x2dd_dis = u(6);
x3dd_dis = u(7);
m = u(8);
g = u(9);
% Jose

A1 = [(2*(x01 - x1)*((x01 - x1)*m^(-1) + (x01 - x1)*mo^(-1)) + 2*(x02 - x2)*((x02 - x2)*m^(-1) + (x02 - x2)*mo^(-1)) +... 
      2*(x03 - x3)*(-((-x03 + x3)*m^(-1)) + (x03 - x3)*mo^(-1)) + ...
      (-2*(x01 - x1)*(-x01 + x41)*mo^(-1) - 2*(x02 - x2)*(-x02 + x42)*mo^(-1) - 2*(x03 - x3)*(-x03 + x43)*mo^(-1))*...
       ((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)^(-0.5)*(L4 - sqrt((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)))/2.,...
   (2*(x01 - x1)*(x01 - x11)*mo^(-1) + 2*(x02 - x12)*(x02 - x2)*mo^(-1) + 2*(x03 - x13)*(x03 - x3)*mo^(-1) +... 
      (-2*(x01 - x11)*(-x01 + x41)*mo^(-1) - 2*(x02 - x12)*(-x02 + x42)*mo^(-1) - 2*(x03 - x13)*(-x03 + x43)*mo^(-1))*...
       ((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)^(-0.5)*(L4 - sqrt((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)))/2.,...
   (2*(x01 - x1)*(x01 - x21)*mo^(-1) + 2*(x02 - x2)*(x02 - x22)*mo^(-1) + 2*(x03 - x12)*(x03 - x3)*mo^(-1) + ...
      (-2*(x01 - x21)*(-x01 + x41)*mo^(-1) - 2*(x02 - x22)*(-x02 + x42)*mo^(-1) - 2*(x03 - x12)*(-x03 + x43)*mo^(-1))*...
       ((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)^(-0.5)*(L4 - sqrt((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)))/2.,...
   (2*(x01 - x1)*(x01 - x31)*mo^(-1) + 2*(x02 - x2)*(x02 - x32)*mo^(-1) + 2*(x03 - x3)*(x03 - x33)*mo^(-1) + ...
      (-2*(x01 - x31)*(-x01 + x41)*mo^(-1) - 2*(x02 - x32)*(-x02 + x42)*mo^(-1) - 2*(x03 - x33)*(-x03 + x43)*mo^(-1))*...
       ((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)^(-0.5)*(L4 - sqrt((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)))/2.,...
   (2*(x01 - x1)*(x01 - x41)*mo^(-1) + 2*(x02 - x2)*(x02 - x42)*mo^(-1) + 2*(x03 - x3)*(x03 - x43)*mo^(-1) - ...
      2*(-(L0*m4^(-1)) - ((-2*(x01 - x41)*(-x01 + x41)*mo^(-1) - 2*(x02 - x42)*(-x02 + x42)*mo^(-1) - ...
              2*(x03 - x43)*(-x03 + x43)*mo^(-1))*((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)^(-0.5))/2.)*...
       (L4 - sqrt((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)))/2.];
       
	   
	   
A2 =[(2*(x01 - x1)*(x01 - x11)*mo^(-1) + 2*(x02 - x12)*(x02 - x2)*mo^(-1) + 2*(x03 - x13)*(x03 - x3)*mo^(-1))/2.,...
   (2*m1^(-1)*(L1)^2 + 2*mo^(-1)*(x01 - x11)^2 + 2*mo^(-1)*(x02 - x12)^2 + 2*mo^(-1)*(x03 - x13)^2)/2.,...
   (2*(x03 - x12)*(x03 - x13)*mo^(-1) + 2*(x01 - x11)*(x01 - x21)*mo^(-1) + 2*(x02 - x12)*(x02 - x22)*mo^(-1))/2.,...
   (2*(x01 - x11)*(x01 - x31)*mo^(-1) + 2*(x02 - x12)*(x02 - x32)*mo^(-1) + 2*(x03 - x13)*(x03 - x33)*mo^(-1))/2.,...
   (2*(x01 - x11)*(x01 - x41)*mo^(-1) + 2*(x02 - x12)*(x02 - x42)*mo^(-1) + 2*(x03 - x13)*(x03 - x43)*mo^(-1))/2.];
    
A3 =[(2*(x01 - x1)*(x01 - x21)*mo^(-1) + 2*(x02 - x2)*(x02 - x22)*mo^(-1) + 2*(x03 - x12)*(x03 - x3)*mo^(-1))/2.,...
   (2*(x03 - x12)*(x03 - x13)*mo^(-1) + 2*(x01 - x11)*(x01 - x21)*mo^(-1) + 2*(x02 - x12)*(x02 - x22)*mo^(-1))/2.,...
   (2*m2^(-1)*(L2)^2 + 2*mo^(-1)*(x03 - x12)^2 + 2*mo^(-1)*(x01 - x21)^2 + 2*mo^(-1)*(x02 - x22)^2)/2.,...
   (2*(x01 - x21)*(x01 - x31)*mo^(-1) + 2*(x02 - x22)*(x02 - x32)*mo^(-1) + 2*(x03 - x12)*(x03 - x33)*mo^(-1))/2.,...
   (2*(x01 - x21)*(x01 - x41)*mo^(-1) + 2*(x02 - x22)*(x02 - x42)*mo^(-1) + 2*(x03 - x12)*(x03 - x43)*mo^(-1))/2];
 
A4 = [(2*(x01 - x1)*(x01 - x31)*mo^(-1) + 2*(x02 - x2)*(x02 - x32)*mo^(-1) + 2*(x03 - x3)*(x03 - x33)*mo^(-1))/2.,...
   (2*(x01 - x11)*(x01 - x31)*mo^(-1) + 2*(x02 - x12)*(x02 - x32)*mo^(-1) + 2*(x03 - x13)*(x03 - x33)*mo^(-1))/2.,...
   (2*(x01 - x21)*(x01 - x31)*mo^(-1) + 2*(x02 - x22)*(x02 - x32)*mo^(-1) + 2*(x03 - x12)*(x03 - x33)*mo^(-1))/2.,...
   (2*m3^(-1)*(L3)^2 + 2*mo^(-1)*(x01 - x31)^2 + 2*mo^(-1)*(x02 - x32)^2 + 2*mo^(-1)*(x03 - x33)^2)/2.,...
   (2*(x01 - x31)*(x01 - x41)*mo^(-1) + 2*(x02 - x32)*(x02 - x42)*mo^(-1) + 2*(x03 - x33)*(x03 - x43)*mo^(-1))/2.];
      
A5 =[sqrt((-x01 + x1)^2 + (-x02 + x2)^2 + (-x03 + x3)^2),0,0,0,-sqrt((x01 - x41)^2 + (x02 - x42)^2 + (x03 - x43)^2)];



B1 = (2*(x01d - x1d)^2 + 2*(x02d - x2d)^2 + 2*(x03d - x3d)^2 - ...
     2*(L4d - ((-2*x01d*(-x01 + x41) - 2*x02d*(-x02 + x42) - 2*x03d*(-x03 + x43))*...
           ((-x01 + x41)^(2) + (-x02 + x42)^(2) + (-x03 + x43)^(2))^(-0.5))/2.)^2 -... 
     2*(T4*(m4)^(-1) - (((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)^(-0.5)*...
           (2*g*(-x03 + x43) + 2*(x01d)^2 + 2*(x02d)^2 + 2*(x03d)^2))/2. +... 
        (((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)^(-1.5)*...
           (-2*x01d*(-x01 + x41) - 2*x02d*(-x02 + x42) - 2*x03d*(-x03 + x43))^2)/4.)*...
      (L4 - sqrt((-x01 + x41)^2 + (-x02 + x42)^2 + (-x03 + x43)^2)))/2.;
	  
B2 =(-2*g*(x03 - x13) - 2*L1*T1*m1^(-1) - 2*(L1d)^2 + 2*(x01d)^2 + 2*(x02d)^2 + 2*(x03d)^2)/2.;
B3 =(-2*g*(x03 - x12) - 2*L2*T2*m2^(-1) - 2*(L2d)^2 + 2*(x01d)^2 + 2*(x02d)^2 + 2*(x03d)^2)/2.;
B4 =(-2*g*(x03 - x33) - 2*L3*T3*m3^(-1) - 2*(L3d)^2 + 2*(x01d)^2 + 2*(x02d)^2 + 2*(x03d)^2)/2.;
B5 = 0;

A = [A1;A2;A3;A4;A5];
B = [B1;B2;B3;B4;B5];

% l est pour lambda

l = -inv(A)*B;
l1 = l(1);
l2 = l(2);
l3 = l(3);
l4 = l(4);
l5 = l(5);
 

% new equations of mouvement 

x1dd = -(x01 - x1)*l1*m^(-1) + x1dd_dis;
x2dd = -(x02 - x2)*l1*m^(-1) + x2dd_dis;
x3dd = -(g*m + x03*l1 - x3*l1)*m^(-1) + x3dd_dis;



x01dd = (x01*l1 - x1*l1 + x01*l2 - x11*l2 + x01*l3 - x21*l3 + x01*l4 - x31*l4 + x01*l5 - x41*l5)*mo^(-1);
x02dd = (x02*l1 - x2*l1 + x02*l2 - x12*l2 + x02*l3 - x22*l3 + x02*l4 - x32*l4 + x02*l5 - x42*l5)*mo^(-1);
x03dd = (-g*mo + x03*l1 - x3*l1 + x03*l2 - x13*l2 + x03*l3 - x12*l3 + x03*l4 - x33*l4 + x03*l5 - x43*l5)*mo^(-1);


L0dd  = pow(L0,-1)*(x01*x01dd + x02*x02dd + x03*x03dd - x01dd*x41 - x02dd*x42 - x03dd*x43 - sq(L0d) + sq(x01d) + sq(x02d) + sq(x03d));
L1dd  = pow(L1,-1)*(x01*x01dd + x02*x02dd + x03*x03dd - x01dd*x11 - x02dd*x12 - x03dd*x13 - sq(L1d) + sq(x01d) + sq(x02d) + sq(x03d));
L2dd  = pow(L2,-1)*(x01*x01dd + x02*x02dd + x03*x03dd - x01dd*x21 - x02dd*x22 - x03dd*x23 - sq(L2d) + sq(x01d) + sq(x02d) + sq(x03d));
L3dd  = pow(L3,-1)*(x01*x01dd + x02*x02dd + x03*x03dd - x01dd*x31 - x02dd*x32 - x03dd*x33 - sq(L3d) + sq(x01d) + sq(x02d) + sq(x03d));
L4dd  = pow(L0,-1)*(x01*x01dd + x02*x02dd + x03*x03dd - x01dd*x41 - x02dd*x42 - x03dd*x43 - sq(L0d) + sq(x01d) + sq(x02d) + sq(x03d)) +... 
        pow(-L0 + L4,-1)*((x01 - x1)*(x01dd - x1dd) + (x02 - x2)*(x02dd - x2dd) + (x03 - x3)*(x03dd - x3dd) - sq(L0d - L4d) + sq(x01d - x1d) + ... 
        sq(x02d - x2d) + sq(x03d - x3d));
     

sys(1)  = x1d;
sys(2)  = x1dd;
sys(3)  = x2d;
sys(4)  = x2dd;
sys(5)  = x3d;
sys(6)  = x3dd;
sys(7)  = x01d;
sys(8)  = x01dd;
sys(9)  = x02d;
sys(10) = x02dd;
sys(11) = x03d;
sys(12) = x03dd;
sys(13) = L0d;
sys(14) = L0dd;
sys(15) = L1d;
sys(16) = L1dd;
sys(17) = L2d;
sys(18) = L2dd;
sys(19) = L3d;
sys(20) = L3dd;
sys(21) = L4d;
sys(22) = L4dd;


% end mdlDerivatives

%
%=============================================================================
% mdlOutputs
% Return the output vector for the S-function
%=============================================================================
%
function sys = mdlOutputs(t,x,u)

sys(1) = x(1);
sys(2) = x(2);
sys(3) = x(3);
sys(4) = x(4);
sys(5) = x(5);
sys(6) = x(6);
sys(7) = x(7);
sys(8) = x(8);
sys(9) = x(9);
sys(10) = x(10);
sys(11) = x(11);
sys(12) = x(12);
sys(13) = x(13);
sys(14) = x(14);
sys(15) = x(15);
sys(16) = x(16);
sys(17) = x(17);
sys(18) = x(18);
sys(19) = x(19);
sys(20) = x(20);
sys(21) = x(21);
sys(22) = x(22);


% end mdlOutputs